/* 
 * File:   Strings_GLCD.c
 * Author: chengo_cc
 *
 * Created on 30 November 2015, 10:51 PM
 */

#include <stdio.h>
#include <stdlib.h>
#include <htc.h>
#include <pic.h>
#include "charset.h"
#include <string.h>
#include "delay.h"
#include "picture.h"
//------------------------------------------------------------------------------
#define port        PORTD
#define rs          RC3         //Register Select
#define en          RC4         //ENable
#define cs2         RC5         //Chip Select
#define cs1         RC6
//------------------------------------------------------------------------------
#define port_dir    TRISD       //Direction: i/o
#define rs_dir      TRISC3
#define en_dir      TRISC4
#define cs2_dir     TRISC5
#define cs1_dir     TRISC6
//------------------------------------------------------------------------------
#define ON          0x3F
#define OFF         0x3E
#define glcdWidth   128      //128 bits since 128 x 64
#define pgWidth     64       //64 bits on chip1 and chip2 each
#define pgLen       8        //8 Bits
#define CLEAR       0x00
#define FILL        0xFF
//------------------------------------------------------------------------------
unsigned char page[8] = {0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF}; //LCD Pages addresses
unsigned int  column = 0;
void latch();
//------------------------------------------------------------------------------
void GLCD_chip(int chip)
{
  switch(chip)
  {
    case 1: cs2 = 1; cs1 = 0; break;  //Selects Chip 1. NB: Both are Active LOW!
    case 2: cs2 = 0; cs1 = 1; break;  //Selects Chip 2
    case 3: cs2 = 0; cs1 = 0; break;  //Selects Both
  }
}
//------------------------------------------------------------------------------
void GLCD_command(char command, int chip)
{
  GLCD_chip(chip);
  rs = 0;                       //Instruction Register
  port = command;
  latch();
}
//------------------------------------------------------------------------------
void GLCD_data(char data, int chip)
{
  GLCD_chip(chip);
  rs = 1;                       //Data Register
  port = data;
  latch();
}
//------------------------------------------------------------------------------
void GLCD_column(int col)
{
  column = col;                      //column is a global variable that keeps track of column infomation
  if( col < pgWidth )
  {
    col = 0x40 | col;                //0x40 represents Column 0
    GLCD_command(col,1);
  }
  else
      if( col < glcdWidth )
      {
        col = 0x40 | (col - pgWidth);
        GLCD_command(col,2);
      }
}
//------------------------------------------------------------------------------
void GLCD_page(int pg)
{
  pg = page[pg];
  GLCD_command(pg,3);
}
//------------------------------------------------------------------------------
void GLCD_startline(char line)
{
  line = 0xC0 | line;	   //0xC0 represents Line 0
  GLCD_command(line,3);    //Write to Chip 1 and 2
}
//------------------------------------------------------------------------------
void GLCD_bitmap(unsigned char *bitmap, int elements)
{
  for(int e = 0 ; e < elements ; e++)
  {
    if( column < pgWidth )
    {
      GLCD_data(bitmap[e],1);       // Write to Chip 1
    }
    else
        if( column < glcdWidth )
        {
          GLCD_column(column);
          GLCD_data(bitmap[e],2);    //Write to Chip 2
        }
    column++;
  }
}
//------------------------------------------------------------------------------
void GLCD_font1(int pg,int col,unsigned char *text)
{
  GLCD_page(pg);
  GLCD_column(col);

  while( *text != '\0' )
  {
    int row = *text++;
    GLCD_bitmap( &Character8x8[row*8] , 8 ); //Pass Address(with index as a multiple of 8) with the array size
  }
}
//------------------------------------------------------------------------------
void GLCD_font2(int pg,int col,unsigned char *text)
{
  GLCD_page(pg);
  GLCD_column(col);
 
  while( *text != '\0' )
  {
    int row = *text++ - 32 ;
    GLCD_bitmap( &font5x7[row*5] , 5);
  }
}
//------------------------------------------------------------------------------
void GLCD_image(unsigned char *image)
{
  for(int pg = 0; pg < pgLen ;pg++)
  {
    GLCD_page(pg);
    GLCD_column(0);
    GLCD_bitmap( &image[pg*glcdWidth] , glcdWidth );
  }
}
//------------------------------------------------------------------------------
void GLCD_fill(char fill)
{
  for(int pg = 0; pg < pgLen; pg++)
  {
    GLCD_page(pg);
    GLCD_column(0);
    for(int col = 0; col < glcdWidth; col++)
    {
      GLCD_bitmap(&fill,1);
    }
  } 
}
//------------------------------------------------------------------------------
void GLCD_init()
{
  port_dir  = 0;
  rs_dir    = 0;
  en_dir    = 0;
  cs1_dir   = 0;
  cs2_dir   = 0;

  GLCD_command(ON,3);
  GLCD_fill(0x00);
}
//------------------------------------------------------------------------------
void latch()
{
  en = 1;
  en = 0;
}
//------------------------------------------------------------------------------
void main()
{
   GLCD_init();

   GLCD_image(truck);
   delay(1000);
   GLCD_fill(CLEAR);
   delay(2000);
   
   GLCD_font1(1,0,"****************");
   GLCD_font1(2,0,"KS0108 GLCD     ");
   GLCD_font2(3,0,"characters and images");
   GLCD_font1(4,0,"****************");
   delay(4000);
 
   while(1)
   {
     for(int ln = 0; ln < 64; ++ln)
     {
       GLCD_startline(ln);
       delay(10);
     }
   }
}
//------------------------------------------------------------------------------